#!/bin/sh
#
# Make_Key_Store.sh
# Make a PKCS12 key store for the REST Process of the Automation Engine.
#
# Author
# Michael A. Lowry <michael_lowry@swissre.com> <michael.lowry@gmail.com>
#
# Version history
# 0.1	MAL	2021.09.02	Initial version
# 0.2	MAL	2023.09.26	Additional checks + added '-deststore pkcs12'
# 0.3	MAL	2023.09.27	Check for existence of target keystore
#

shopt -s extglob
VERBOSE_MODE="yes"
SCRIPT_NAME=$(basename $0)
SCRIPT_VERSION="0.3"
Temp_Dir="/tmp"
echo "$SCRIPT_NAME v$SCRIPT_VERSION"
usage () {
cat <<EOF

NAME
  $SCRIPT_NAME

SYNOPSIS
  Create a PKCS12 key store for the REST Process of the Automation Engine server

DESCRIPTION
    $SCRIPT_NAME <input_cert_chain> <password> <output_key_store>

  Print this usage stagement
    $SCRIPT_NAME 

OPTIONS
  <input_cert_chain>
        The PKCS#12 certificate chain provided by the Certificate Authority (CA).

  <password>
        The password used to encrypt the private key in the certificate chain.
        This password will also be used as the key store passowrd.

  <output_key_store>
        The name of the key store file to write.

EXAMPLE
 $SCRIPT_NAME cert_uc4.mycompany.com_1DC39EF000000000EF6A.p12 CDJFJ3kdkj3dSPn keystore

EOF
        exit 1
}

if [[ $# -ne 3 ]]; then usage; fi
Java_Security_Config="${JAVA_ROOT}/lib/security/java.security"
if [[ ! -f "${Java_Security_Config}" ]]; then
	echo "${Java_Security_Config} does not exist. Please create this file and add the following line: "
	echo "keystore.type=pkcs12"
	echo "Then, rerun this script."
fi
Keystore_Type=$(awk -F= '$1=="keystore.type" {print $2}' "${Java_Security_Config}" | dos2unix )
echo "keystore.type in ${Java_Security_Config}: ${Keystore_Type}"
if [[ "${Keystore_Type}" != "pkcs12" ]]; then
	echo "keystore.type in ${Java_Security_Config} is set to ${Keystore_Type}. It must be set to pkcs12."
	exit 1
fi
echo
Input_Cert_Chain=$1
Password=$2
Output_Key_Store=$3
Temp_Info="${Temp_Dir}/Temp_Info.$$.txt"
Temp_Private_Key="${Temp_Dir}/Temp_Private_Key.$$.txt"
Temp_Certificate="${Temp_Dir}/Temp_Certificate.$$.txt"
Temp_Cert_Chain="${Temp_Dir}/Temp_Cert_Chain.$$.txt"
if [[ ! -f "${Input_Cert_Chain}" ]]; then echo "Certificate chain does not exist or is not readable: ${Input_Cert_Chain}"; exit 1; fi
if [[ -f "${Output_Key_Store}" ]]; then echo "Target keystore ${Output_Key_Store} already exists."; exit 1; fi

echo "### Validating contents of PKCS12 trust chain."
echo "openssl pkcs12 -in ${Input_Cert_Chain} -info -passin pass:XXXX -passout pass:XXXX"
openssl pkcs12 -in ${Input_Cert_Chain} -info -passin pass:${Password} -passout pass:${Password} > "${Temp_Info}" 2>&1
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi
grep -i subject "${Temp_Info}"
echo "See "${Temp_Info}" for additional details."
echo

echo "### Extracting private key from trust chain."
echo "openssl pkcs12 -in ${Input_Cert_Chain} -nocerts -out ${Temp_Private_Key} -passin pass:XXXX -passout pass:XXXX"
openssl pkcs12 -in "${Input_Cert_Chain}" -nocerts -out "${Temp_Private_Key}" -passin pass:${Password} -passout pass:${Password}
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi
echo

echo "### Extracting client certificate from trust chain."
echo "openssl pkcs12 -in ${Input_Cert_Chain} -clcerts -nokeys -out ${Temp_Certificate} -passin pass:XXXX -passout pass:XXXX"
openssl pkcs12 -in "${Input_Cert_Chain}" -clcerts -nokeys -out "${Temp_Certificate}" -passin pass:${Password} -passout pass:${Password}
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi
echo

echo "### Creating temporary certificate chain from private key & certificate."
echo "openssl pkcs12 -inkey ${Temp_Private_Key} -in ${Temp_Certificate} -export -out ${Temp_Cert_Chain} -passin pass:XXXX -passout pass:XXXX"
openssl pkcs12 -inkey "${Temp_Private_Key}" -in "${Temp_Certificate}" -export -out "${Temp_Cert_Chain}" -passin pass:${Password} -passout pass:${Password}
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi
echo

echo "### Creating trust store from temporary certificate chain."
echo "keytool -importkeystore -srckeystore ${Temp_Cert_Chain} -destkeystore ${Output_Key_Store} -srcstoretype pkcs12 -deststoretype pkcs12 -alias "1" -destalias "jetty" -srcstorepass XXXX -deststorepass XXXX"
keytool -importkeystore -srckeystore "${Temp_Cert_Chain}" -destkeystore "${Output_Key_Store}" -srcstoretype pkcs12 -deststoretype pkcs12 -alias "1" -destalias "jetty" -srcstorepass ${Password} -deststorepass ${Password}
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi

if [[ $VERBOSE_MODE == "no" ]]; then echo "Removing temporary files."; rm  "${Temp_Info}" "${Temp_Private_Key}" "${Temp_Certificate}" "${Temp_Cert_Chain}"; fi
echo

echo "### Listing contents of keystore"
echo "keytool -list -v -keystore ${Output_Key_Store} -storepass XXXX"
keytool -list -v -keystore "${Output_Key_Store}" -storepass ${Password}
RC=$?
if [[ $RC -ne 0 ]]; then echo "Error encountered (RC=$RC). Exiting."; exit 1; fi
echo

echo "Keystore created successfully."
